message Attribute
 {
  string name = 0
 repeated  string dims = 0
 repeated  string shape = 0
  anyType value = 0
  anyType defaultValue = 0
  string doc = 0
  boolean required = 0
 repeated  string dtype = 0
 }
message CompoundDtype
 {
  string name = 0
  string doc = 0
 repeated  string dtype = 0
 }
message Dataset
 {
  string neurodataTypeDef = 0
  string neurodataTypeInc = 0
  string name = 0
  string defaultName = 0
 repeated  string dims = 0
 repeated  string shape = 0
  anyType value = 0
  anyType defaultValue = 0
  string doc = 0
  string quantity = 0
  boolean linkable = 0
 repeated  attribute attributes = 0
 repeated  string dtype = 0
 }
message Datasets
 {
 repeated  dataset datasets = 0
 }
message Group
 {
  string neurodataTypeDef = 0
  string neurodataTypeInc = 0
  string name = 0
  string defaultName = 0
  string doc = 0
  string quantity = 0
  boolean linkable = 0
 repeated  attribute attributes = 0
 repeated  dataset datasets = 0
 repeated  group groups = 0
 repeated  link links = 0
 }
message Groups
 {
 repeated  group groups = 0
 }
message Link
 {
  string name = 0
  string doc = 0
  string targetType = 0
  string quantity = 0
 }
message Namespace
 {
  string doc = 0
  string name = 0
  string fullName = 0
  string version = 0
  datetime date = 0
 repeated  string author = 0
 repeated  string contact = 0
 repeated  schema schema = 0
 }
message Namespaces
 {
 repeated  namespace namespaces = 0
 }
message ReferenceDtype
 {
  string targetType = 0
  reftypeOptions reftype = 0
 }
message Schema
 {
  string source = 0
  string namespace = 0
  string title = 0
 repeated  string neurodataTypes = 0
  string doc = 0
 }
